﻿using System;
using System.Collections.ObjectModel;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.ComponentModel.DataAnnotations;


namespace GridSample
{
    public class MyDataObject : INotifyPropertyChanged, INotifyDataErrorInfo
    {
        #region Private members

        private int _int;
        private string _string;

        #endregion // Private members

        #region Public Properties
        [Range(0,1000)]
        public int Int
        {
            get
            {
                return this._int;
            }
            set
            {
                this.ValidateProperty("Int", value);

                if (value != this._int)
                {
                    this._int = value;
                    RaisePropertyChanged("Int");
                }
            }
        }

        public string String
        {
            get
            {
                return this._string;
            }
            set
            {
                if (value != this._string)
                {
                    this._string = value;
                    RaisePropertyChanged("String");
                }
            }
        }

        #endregion // Public Properties

        #region Constructors

        public MyDataObject(int index)
        {
            this.Int = index;
            this.String = "Item " + index;
        }

        #endregion // Constructors

        #region INotifyPropertyChanged Members

        public event PropertyChangedEventHandler PropertyChanged;

        private void RaisePropertyChanged(string propertyName)
        {
            if (this.PropertyChanged != null)
            {
                this.PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }

        #endregion // INotifyPropertyChanged Members

        private readonly IDictionary<string, IList<string>> _errors = new Dictionary<string, IList<string>>();

        public event EventHandler<DataErrorsChangedEventArgs> ErrorsChanged;

        public System.Collections.IEnumerable GetErrors(string propertyName)
        {
            if (_errors.ContainsKey(propertyName))
            {
                IList<string> propertyErrors = _errors[propertyName];

                foreach (string propertyError in propertyErrors)
                {
                    yield return propertyError;
                }
            }

            yield break;
        }

        public bool HasErrors
        {
            get
            {
                return _errors.Count > 0;
            }
        }

        protected void NotifyErrorsChanged(string propertyName)
        {
            if (ErrorsChanged != null)
                ErrorsChanged(this, new DataErrorsChangedEventArgs(propertyName));
        }

        protected void ValidateProperty(string propertyName, object value)
        {
            MyDataObject objectToValidate = this;
            var results = new List<ValidationResult>();
            bool isValid = Validator.TryValidateProperty(
                value,
                new ValidationContext(objectToValidate, null, null)
                {
                    MemberName = propertyName
                },
                results);

            if (isValid)
                RemoveErrorsForProperty(propertyName);
            else
                AddErrorsForProperty(propertyName, results);

            NotifyErrorsChanged(propertyName);
        }

        private void AddErrorsForProperty(string propertyName, IEnumerable<ValidationResult> validationResults)
        {
            RemoveErrorsForProperty(propertyName);
            _errors.Add(propertyName, validationResults.Select(vr => vr.ErrorMessage).ToList());
        }

        private void RemoveErrorsForProperty(string propertyName)
        {
            if (_errors.ContainsKey(propertyName))
            {
                _errors.Remove(propertyName);
            }
        }
    }

    public class MyData : ObservableCollection<MyDataObject>
    {
        public MyData()
        {
            FillDataCollection(this, 5);
        }

        public static void FillDataCollection(IList<MyDataObject> collection, int rowCount)
        {
            for (int i = 0; i < rowCount; i++)
            {
                collection.Add(new MyDataObject(i)
                {
                    String = "Item " + i
                });
            }
        }
    }
}
